# encoding: utf-8
#
# Copyright (C) 2020 willipink.eu
# Author Moritz Münch moritzmuench@mailbox.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.



from datetime import datetime

from django.http import HttpResponse
from django.template import loader
from django.template.defaultfilters import register
from django.views import generic
from django.shortcuts import redirect

from .models import Country, Stamp, Coin, User



class IndexView(generic.ListView):
    template_name = 'coinc/index.html'
    context_object_name = 'countrys'

    def get_queryset(self):
        ''' return all countries ordered by name '''
        return Country.objects.order_by('name_iso')



def detail_country(request, country):
    template = loader.get_template('coinc/country.html')
    c = Country.objects.get(name_iso=country)
    return HttpResponse(template.render(show_country(c, single_country=True), request))



@register.inclusion_tag('coinc/navigation.html')
def show_navigation(countrys):
    return {'countrys': countrys}



@register.inclusion_tag('coinc/country.html')
def show_country(country, single_country=False):
    c = {
        'name_iso': country.name_iso,
        'name': country.name,
        'comment': country.comment,
        'years': {}
    }

    irregular_stamps = {}
    stamps = Stamp.objects.filter(country=country.name_iso).values(
        'name_short', 'years')

    if stamps:
        temp_stamps = {}

        for stamp in stamps:
            stamp_name_short = stamp['name_short']

            if stamp['years'] == "":
                temp_stamps[stamp_name_short] = {}
            else:
                for year in stamp['years'].split(","):
                    irregular_stamps[year] = {}
                    irregular_stamps[year][stamp_name_short] = {}
                temp_stamps[''] = {}

        stamps = temp_stamps
    else:
        stamps = {'': {}}

    year_now = datetime.now().year
    for year in [str(i) for i in range(country.euro_member_since, year_now + 1)]:
        stamps_per_year = stamps

        if irregular_stamps and year in irregular_stamps:
            for irregular_stamp, value in irregular_stamps[year].items():
                stamps_per_year[irregular_stamp] = {}

        # TODO this is not really a fix...
        elif country.name_iso == 'gr':
            stamps_per_year = {'': {}}

        year_short = year[2:4]
        c['years'][year_short] = {}

        for stamp in stamps_per_year:
            c['years'][year_short][stamp] = {'name': stamp}

        stamps_per_year = {}

    return {
        'country': c,
        'single_country': single_country,
        'users': User.objects.order_by('name').values('name','color')
    }



@register.inclusion_tag('coinc/year.html')
def show_year(country, year):
    return {
        'year': country['years'][year],
        'year_short': year,
        'values': [1, 2, 5, 10, 20, 50, 100, 200, 201, 202, 203],
        'country': country['name_iso']
    }



@register.inclusion_tag('coinc/coin.html')
def show_coin(country, year, stamp, value):
    ''' @param country: Country.name_iso
        @param year: int YYYY
        @param stamp: stamp_id
        @param value: int e [1,2,5,10,20,50,100,200,201,202,203]
    '''
    # TODO: fix this before the end of 2098
    year = int(year)
    if year == 99:
        year += 1900
    else:
        year += 2000

    if stamp == "":
        stamp = None

    try:
        coin = Coin.objects.get(country=country, year=year, stamp=stamp, value=value)
    except Coin.DoesNotExist:
        return {}

    if coin.buy_only:
        td_class = 'buy_only'
    elif coin.circulation and coin.circulation < 1500000:
        td_class = 'rare'
    elif coin.noexist:
        td_class = 'noexist'
    elif coin.value in [10, 20, 50]:
        td_class = 'brass'

    if coin.in_collector:
        td_class += ' in_collector'
    elif coin.checked:
        td_class += ' coin_checked'

    if coin.found_by:
        div_class = f"found {coin.found_by}"
        marker = 'x'

    return {
        'marker': marker,
        'td_class': td_class,
        'div_class': div_class,
    }


def add_user(request, username, color):
    ''' add a user '''
    error = False
    message = ''

    if username and color:
        try:
            existing_user = User.objects.get(name=username)
            error = True
            message = f"'{username}' ist schon vergeben."

        except User.DoesNotExist:
#             u = User(name=username, color=color)
#             u.save()
            message = f"Gespeichert"
    else:
        error = True
        message = 'Du musst einen Namen und Farbe angeben.'
    import json
    return HttpResponse(json.dumps({ 'error': error, 'message': message }))


# TODO
def statistic(request):
    return HttpResponse('Statistik')
